@@ -123,6 +123,11 @@ static char pyferretStartDocstring[] =
     "    metaname = <string>: filename for Ferret graphics (default empty) \n"
     "    unmapped = <bool>: hide the graphics viewer? (default False) \n"
     "    quiet = <bool>: do not print the ferret header? (default False) \n"
+    "    linebuffer = <bool>: use line buffering for stdout and stderr? (default False) \n"
+    "            Note: \n"
+    "                the enviroment variable GFORTRAN_UNBUFFERED_PRECONNECTED \n"
+    "                needs to be set to 1 in order to unbuffer the Fortran \n"
+    "                units for output and error messages\n"
     "\n"
     "Returns: \n"
     "    True is successful \n"
@@ -135,7 +140,8 @@ static char pyferretStartDocstring[] =
 static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
 {
     static char *argNames[] = {"memsize", "journal", "verify", "restrict", "server",
-                               "metaname", "transparent", "unmapped", "quiet", NULL};
+                               "metaname", "transparent", "unmapped", "quiet", 
+                               "linebuffer", NULL};
     double mwMemSize = 25.6;
     PyObject *pyoJournal = NULL;
     PyObject *pyoVerify = NULL;
@@ -144,6 +150,7 @@ static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
     PyObject *pyoTransparent = NULL;
     PyObject *pyoUnmapped = NULL;
     PyObject *pyoQuiet = NULL;
+    PyObject *pyoLineBuffer = NULL;
     char *metaname = NULL;
     int journalFlag = 1;
     int verifyFlag = 1;
@@ -152,6 +159,7 @@ static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
     int transparentFlag = 0;
     int unmappedFlag = 0;
     int quietFlag = 0;
+    int lineBufferFlag = 0;
     int pplMemSize;
     size_t blksiz;
     int status;
@@ -169,12 +177,12 @@ static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
     import_array1(NULL);
 
     /* Parse the arguments, checking if an Exception was raised */
-    if ( ! PyArg_ParseTupleAndKeywords(args, kwds, "|dO!O!O!O!sO!O!O!",
+    if ( ! PyArg_ParseTupleAndKeywords(args, kwds, "|dO!O!O!O!sO!O!O!O!",
                  argNames, &mwMemSize, &PyBool_Type, &pyoJournal,
                  &PyBool_Type, &pyoVerify, &PyBool_Type, &pyoRestrict,
                  &PyBool_Type, &pyoServer, &metaname,
                  &PyBool_Type, &pyoTransparent, &PyBool_Type, &pyoUnmapped,
-                 &PyBool_Type, &pyoQuiet) )
+                 &PyBool_Type, &pyoQuiet, &PyBool_Type, &pyoLineBuffer) )
         return NULL;
 
     /* Interpret the booleans - Py_False and Py_True are singleton non-NULL objects, so just use == */
@@ -192,10 +200,18 @@ static PyObject *pyferretStart(PyObject *self, PyObject *args, PyObject *kwds)
         unmappedFlag = 1;
     if ( pyoQuiet == Py_True )
         quietFlag = 1;
+    if ( pyoLineBuffer == Py_True )
+        lineBufferFlag = 1;
 
     if ( metaname[0] == '\0' )
         metaname = NULL;
 
+    /* Deal with line buffering */
+    if ( lineBufferFlag != 0 ) {
+        /* Set line buffering on stdout and stderr; ignore failures */
+        setvbuf(stdout, NULL, _IOLBF, BUFSIZ);
+        setvbuf(stderr, NULL, _IOLBF, BUFSIZ);
+    }
     /* Deal with the restrict and server flags right away */
     if ( restrictFlag != 0 )
         set_secure();
@@ -538,7 +554,7 @@ static char pyferretGetDataDocstring[] =
     "Returns: \n"
     "    A tuple containing: \n"
     "        a NumPy float64 ndarray containing a copy of the numeric data requested, \n"
-    "        a NumPy float64 ndarray containing the bad-data-flag value(s) for the data, \n"
+    "        a NumPy float64 ndarray containing the bad-data-flag value for the data, \n"
     "        a string giving the units for the data \n"
     "        a tuple of six integers giving the AXISTYPE codes of the axes, \n"
     "        a tuple of six strings giving the names of the axes, \n"
@@ -643,7 +659,7 @@ static PyObject *pyferretGetData(PyObject *self, PyObject *args, PyObject *kwds)
      * Note: if MAX_FERRET_NDIM changes, this needs editing.
      */
     ferdata = ferMemory + arraystart;
-    npydata = PyArray_DATA(data_ndarray);
+    npydata = (double *)PyArray_DATA(data_ndarray);
     q = 0;
     for (n = 0; n < (int)(shape[5]); n++) {
       for (m = 0; m < (int)(shape[4]); m++) {
@@ -672,7 +688,7 @@ static PyObject *pyferretGetData(PyObject *self, PyObject *args, PyObject *kwds)
        Py_DECREF(data_ndarray);
        return NULL;
     }
-    npydata = PyArray_DATA(badval_ndarray);
+    npydata = (double *)PyArray_DATA(badval_ndarray);
     npydata[0] = badval;
 
     /* Create the axis coordinates array objects */
@@ -746,22 +762,22 @@ static PyObject *pyferretGetData(PyObject *self, PyObject *args, PyObject *kwds)
             /* set the axis units to the name of the calendar */
             switch( calendar_type ) {
             case CALTYPE_NONE:
-                strcpy(axis_units[k], "CALTYPE_NONE");
+                strcpy(axis_units[k], CALTYPE_NONE_STR);
                 break;
             case CALTYPE_360DAY:
-                strcpy(axis_units[k], "CALTYPE_360DAY");
+                strcpy(axis_units[k], CALTYPE_360DAY_STR);
                 break;
             case CALTYPE_NOLEAP:
-                strcpy(axis_units[k], "CALTYPE_NOLEAP");
+                strcpy(axis_units[k], CALTYPE_NOLEAP_STR);
                 break;
             case CALTYPE_GREGORIAN:
-                strcpy(axis_units[k], "CALTYPE_GREGORIAN");
+                strcpy(axis_units[k], CALTYPE_GREGORIAN_STR);
                 break;
             case CALTYPE_JULIAN:
-                strcpy(axis_units[k], "CALTYPE_JULIAN");
+                strcpy(axis_units[k], CALTYPE_JULIAN_STR);
                 break;
             case CALTYPE_ALLLEAP:
-                strcpy(axis_units[k], "CALTYPE_ALLLEAP");
+                strcpy(axis_units[k], CALTYPE_ALLLEAP_STR);
                 break;
             default:
                 sprintf(errmsg, "Unexpected calendar type of %d", calendar_type);
@@ -902,7 +918,7 @@ static PyObject *pyferretPutData(PyObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
     /* Just get bdfval from the data in bdfval_ndarray */
-    bdfval = ((double *) PyArray_DATA(bdfval_ndarray))[0];
+    bdfval = ((double *)PyArray_DATA(bdfval_ndarray))[0];
 
     /* Get the axis types out of the tuple */
     axis_types_tuple = PySequence_Fast(axis_types_tuple, "axis_types is not a tuple or list");
@@ -1016,7 +1032,7 @@ static PyObject *pyferretPutData(PyObject *self, PyObject *args, PyObject *kwds)
                 Py_DECREF(axis_coords_tuple);
                 return NULL;
             }
-            axis_coords[k] = PyArray_DATA(seqitem);
+            axis_coords[k] = (int *)PyArray_DATA(seqitem);
             get_axis_num_(&(axis_nums[k]), &(axis_starts[k]), &(axis_ends[k]), axis_names[k],
                           axis_units[k], axis_coords[k], &(num_coords[k]), &(axis_types[k]),
                           errmsg, &len_errmsg, strlen(axis_names[k]), strlen(axis_units[k]), 2048);
@@ -1049,22 +1065,22 @@ static PyObject *pyferretPutData(PyObject *self, PyObject *args, PyObject *kwds)
                 return NULL;
             }
             num_coords[k] /= 6;
-            if ( strcmp(axis_units[k], "CALTYPE_NONE") == 0 ) {
+            if ( strcmp(axis_units[k], CALTYPE_NONE_STR) == 0 ) {
                 calendar_type = CALTYPE_NONE;
             }
-            else if ( strcmp(axis_units[k], "CALTYPE_360DAY") == 0 ) {
+            else if ( strcmp(axis_units[k], CALTYPE_360DAY_STR) == 0 ) {
                 calendar_type = CALTYPE_360DAY;
             }
-            else if ( strcmp(axis_units[k], "CALTYPE_NOLEAP") == 0 ) {
+            else if ( strcmp(axis_units[k], CALTYPE_NOLEAP_STR) == 0 ) {
                 calendar_type = CALTYPE_NOLEAP;
             }
-            else if ( strcmp(axis_units[k], "CALTYPE_GREGORIAN") == 0 ) {
+            else if ( strcmp(axis_units[k], CALTYPE_GREGORIAN_STR) == 0 ) {
                 calendar_type = CALTYPE_GREGORIAN;
             }
-            else if ( strcmp(axis_units[k], "CALTYPE_JULIAN") == 0 ) {
+            else if ( strcmp(axis_units[k], CALTYPE_JULIAN_STR) == 0 ) {
                 calendar_type = CALTYPE_JULIAN;
             }
-            else if ( strcmp(axis_units[k], "CALTYPE_ALLLEAP") == 0 ) {
+            else if ( strcmp(axis_units[k], CALTYPE_ALLLEAP_STR) == 0 ) {
                 calendar_type = CALTYPE_ALLLEAP;
             }
             else {
@@ -1072,7 +1088,7 @@ static PyObject *pyferretPutData(PyObject *self, PyObject *args, PyObject *kwds)
                 Py_DECREF(axis_coords_tuple);
                 return NULL;
             }
-            axis_coords[k] = PyArray_DATA(seqitem);
+            axis_coords[k] = (int *)PyArray_DATA(seqitem);
             get_time_axis_num_(&(axis_nums[k]), &(axis_starts[k]), &(axis_ends[k]),
                                axis_names[k], &calendar_type, axis_coords[k], &(num_coords[k]),
                                errmsg, &len_errmsg, strlen(axis_names[k]), 2048);
@@ -1124,6 +1140,332 @@ static PyObject *pyferretPutData(PyObject *self, PyObject *args, PyObject *kwds)
     return Py_None;
 }
 
+static char pyferretGetStrDataDocstring[] =
+    "Returns the String data array described in the argument. \n"
+    "\n"
+    "Required arguments: \n"
+    "    name = <string>: the name of the String data array to return \n"
+    "\n"
+    "Optional arguments: \n"
+    "    (none) \n"
+    "\n"
+    "Returns: \n"
+    "    A tuple containing: \n"
+    "        a NumPy String ndarray containing a copy of the numeric data requested, \n"
+    "        a NumPy String ndarray containing the bad-data-flag value for the data, \n"
+    "        a tuple of six integers giving the AXISTYPE codes of the axes, \n"
+    "        a tuple of six strings giving the names of the axes, \n"
+    "        a tuple of six strings giving the units of a non-calendar-time data axis, or \n"
+    "                                       the CALTYPE_ calendar name of a calendar-time axis, \n"
+    "        a tuple of six ndarrays giving the coordinates for the data axes \n"
+    "            (ndarray of N doubles for non-calendar-time, non-normal axes, \n"
+    "             ndarray of (N,6) integers for calendar-time axes, or \n"
+    "             None for normal axes) \n"
+    "\n"
+    "Raises: \n"
+    "    ValueError if the data name is invalid \n"
+    "    MemoryError if Ferret has not been started or has been stopped \n";
+
+static PyObject *pyferretGetStrData(PyObject *self, PyObject *args, PyObject *kwds)
+{
+    static char   *argNames[] = {"name", NULL};
+    char          *name;
+    int            lendataname;
+    char           dataname[1024];
+    int            arraystart;
+    int            memlo[MAX_FERRET_NDIM], memhi[MAX_FERRET_NDIM];
+    int            steplo[MAX_FERRET_NDIM], stephi[MAX_FERRET_NDIM], incr[MAX_FERRET_NDIM];
+    AXISTYPE       axis_types[MAX_FERRET_NDIM];
+    char           errmsg[2112];
+    int            lenerrmsg;
+    int            i, j, k, l, m, n, q;
+    npy_intp       shape[MAX_FERRET_NDIM];
+    npy_intp       new_shape[2];
+    int            strides[MAX_FERRET_NDIM];
+    PyObject      *data_ndarray;
+    void         **ferdata;
+    char          *strptr;
+    int            maxstrlen;
+    int            thisstrlen;
+    PyArray_Descr *strarraydescript;
+    char          *npydata;
+    PyObject      *badval_ndarray;
+    PyObject      *axis_coords[MAX_FERRET_NDIM];
+    char           axis_units[MAX_FERRET_NDIM][64];
+    char           axis_names[MAX_FERRET_NDIM][64];
+    CALTYPE        calendar_type;
+
+    /* If not initialized, raise a MemoryError */
+    if ( ! ferretInitialized ) {
+        PyErr_SetString(PyExc_MemoryError, "Ferret not started");
+        return NULL;
+    }
+
+    /* Parse the arguments, checking if an Exception was raised */
+    if ( ! PyArg_ParseTupleAndKeywords(args, kwds, "s", argNames, &name) )
+        return NULL;
+
+    /* Make a copy of dataname just to be sure it isn't altered */
+    lendataname = strlen(name);
+    if ( lendataname > 1020 ) {
+        PyErr_SetString(PyExc_ValueError, "name too long");
+        return NULL;
+    }
+    strcpy(dataname, name);
+
+    /*
+     * Retrieve the memory parameters describing the data array requested.
+     * Assumes Unix standard for passing strings to Fortran (appended array lengths).
+     */
+    get_str_data_array_params_(dataname, &lendataname, ferMemory, &arraystart, 
+                               memlo, memhi, steplo, stephi, incr, axis_types,
+                               errmsg, &lenerrmsg, 1024, 64, 2112);
+    if ( lenerrmsg > 0 ) {
+        errmsg[lenerrmsg] = '\0';
+        PyErr_SetString(PyExc_ValueError, errmsg);
+        return NULL;
+    }
+
+    /* Get the shape of the array */
+    for (k = 0; k < MAX_FERRET_NDIM; k++)
+        shape[k] = (npy_intp) ((stephi[k] - steplo[k] + incr[k]) / (incr[k]));
+
+    /* Get the strides through the memory (as a double *) */
+    strides[0] = 1;
+    for (k = 1; k < MAX_FERRET_NDIM; k++)
+        strides[k] = strides[k-1] * (memhi[k-1] - memlo[k-1] + 1);
+
+    /* Get the actual starting point in the array */
+    for (k = 0; k < MAX_FERRET_NDIM; k++)
+        arraystart += (strides[k]) * (steplo[k] - memlo[k]);
+
+    /* Convert to strides through places in memory to be read */
+    for (k = 0; k < MAX_FERRET_NDIM; k++)
+        strides[k] *= incr[k];
+
+    /* 
+     * Note: for string arrays, each "double" is a pointer 
+     * to allocated memory containing a null-terminated string;
+     * or null for missing strings.
+     */
+
+    /* Get the maximum string length of all the strings in this array */
+    ferdata = (void **) (ferMemory + arraystart);
+    /* Use STRING_MISSING_VALUE for missing strings */
+    maxstrlen = (int)(strlen(STRING_MISSING_VALUE) + 1);
+    for (n = 0; n < (int)(shape[5]); n++) {
+      for (m = 0; m < (int)(shape[4]); m++) {
+        for (l = 0; l < (int)(shape[3]); l++) {
+          for (k = 0; k < (int)(shape[2]); k++) {
+            for (j = 0; j < (int)(shape[1]); j++) {
+              for (i = 0; i < (int)(shape[0]); i++) {
+                strptr = (char *) ferdata[ i * strides[0] + 
+                                           j * strides[1] + 
+                                           k * strides[2] + 
+                                           l * strides[3] +
+                                           m * strides[4] +
+                                           n * strides[5] ];
+                if ( strptr != NULL ) {
+                  /* add one so always null-terminated */
+                  thisstrlen = (int)(strlen(strptr) + 1);
+                  if ( maxstrlen < thisstrlen ) {
+                    maxstrlen = thisstrlen;
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+
+    /* Create a new NumPy String ndarray (Fortran ordering) with the same shape */
+    strarraydescript = PyArray_DescrNewFromType(NPY_STRING);
+    strarraydescript->elsize = maxstrlen;
+    data_ndarray = PyArray_Empty(MAX_FERRET_NDIM, shape, strarraydescript, 1);
+    if ( data_ndarray == NULL ) {
+        return NULL;
+    }
+    /* PyArray_Empty steals the reference to strarraydescript so do not free or reuse */
+
+    /*
+     * Assign the data in the new ndarray.
+     * Note: if MAX_FERRET_NDIM changes, this needs editing.
+     */
+    npydata = (char *)PyArray_DATA(data_ndarray);
+    q = 0;
+    for (n = 0; n < (int)(shape[5]); n++) {
+      for (m = 0; m < (int)(shape[4]); m++) {
+        for (l = 0; l < (int)(shape[3]); l++) {
+          for (k = 0; k < (int)(shape[2]); k++) {
+            for (j = 0; j < (int)(shape[1]); j++) {
+              for (i = 0; i < (int)(shape[0]); i++) {
+                strptr = (char *) ferdata[ i * strides[0] + 
+                                           j * strides[1] + 
+                                           k * strides[2] + 
+                                           l * strides[3] +
+                                           m * strides[4] +
+                                           n * strides[5] ];
+                if ( strptr == NULL ) {
+                   strptr = STRING_MISSING_VALUE;
+                }
+                /* because of +1 to strlen, these will always be null-terminated */
+                strncpy(&(npydata[q]), strptr, maxstrlen);
+                q += maxstrlen;
+              }
+            }
+          }
+        }
+      }
+    }
+
+    /* Create a new NumPy String ndarray with the bad-data-flag value */
+    new_shape[0] = 1;
+    strarraydescript = PyArray_DescrNewFromType(NPY_STRING);
+    strarraydescript->elsize = maxstrlen;
+    badval_ndarray = PyArray_Empty(1, new_shape, strarraydescript, 0);
+    if ( badval_ndarray == NULL ) {
+       Py_DECREF(data_ndarray);
+       return NULL;
+    }
+    /* PyArray_Empty steals the reference to strarraydescript so do not free or reuse */
+    npydata = (char *)PyArray_DATA(badval_ndarray);
+    strncpy(npydata, STRING_MISSING_VALUE, maxstrlen);
+
+    /* Create the axis coordinates array objects */
+    for (k = 0; k < MAX_FERRET_NDIM; k++) {
+        switch( axis_types[k] ) {
+        case AXISTYPE_LONGITUDE:
+        case AXISTYPE_LATITUDE:
+        case AXISTYPE_LEVEL:
+        case AXISTYPE_CUSTOM:
+        case AXISTYPE_ABSTRACT:
+            /* array of doubles, possibly with a units string */
+            axis_coords[k] = PyArray_SimpleNew(1, &(shape[k]), NPY_DOUBLE);
+            if ( axis_coords[k] == NULL ) {
+                while ( k > 0 ) {
+                    k--;
+                    Py_DECREF(axis_coords[k]);
+                }
+                Py_DECREF(badval_ndarray);
+		Py_DECREF(data_ndarray);
+                return NULL;
+            }
+            /* get the coordinates and the units string */
+            q = k+1;
+            j = (int) shape[k];
+            get_data_array_coords_((double *)PyArray_DATA(axis_coords[k]), axis_units[k],
+                                   axis_names[k], &q, &j, errmsg, &lenerrmsg, 64, 64, 2112);
+            if ( lenerrmsg > 0 ) {
+                errmsg[lenerrmsg] = '\0';
+                PyErr_SetString(PyExc_ValueError, errmsg);
+                Py_DECREF(axis_coords[k]);
+                while ( k > 0 ) {
+                    k--;
+                    Py_DECREF(axis_coords[k]);
+                }
+                Py_DECREF(badval_ndarray);
+                Py_DECREF(data_ndarray);
+                return NULL;
+            }
+            break;
+        case AXISTYPE_TIME:
+            /* array of 6-tuples of integers in C order, so: [N][6] in C or (6,N) in Fortran */
+            new_shape[0] = shape[k];
+            new_shape[1] = 6;
+            axis_coords[k] = PyArray_SimpleNew(2, new_shape, NPY_INT);
+            if ( axis_coords[k] == NULL ) {
+                while ( k > 0 ) {
+                    k--;
+                    Py_DECREF(axis_coords[k]);
+                }
+                Py_DECREF(badval_ndarray);
+                Py_DECREF(data_ndarray);
+                return NULL;
+            }
+            /* get the time coordinate integers */
+            q = k+1;
+            j = (int) shape[k];
+            get_data_array_time_coords_((int (*)[6])PyArray_DATA(axis_coords[k]), &calendar_type, axis_names[k],
+                                        &q, &j, errmsg, &lenerrmsg, 64, 2112);
+            if ( lenerrmsg > 0 ) {
+                errmsg[lenerrmsg] = '\0';
+                PyErr_SetString(PyExc_ValueError, errmsg);
+                Py_DECREF(axis_coords[k]);
+                while ( k > 0 ) {
+                    k--;
+                    Py_DECREF(axis_coords[k]);
+                }
+                Py_DECREF(badval_ndarray);
+                Py_DECREF(data_ndarray);
+                return NULL;
+            }
+            /* set the axis units to the name of the calendar */
+            switch( calendar_type ) {
+            case CALTYPE_NONE:
+                strcpy(axis_units[k], CALTYPE_NONE_STR);
+                break;
+            case CALTYPE_360DAY:
+                strcpy(axis_units[k], CALTYPE_360DAY_STR);
+                break;
+            case CALTYPE_NOLEAP:
+                strcpy(axis_units[k], CALTYPE_NOLEAP_STR);
+                break;
+            case CALTYPE_GREGORIAN:
+                strcpy(axis_units[k], CALTYPE_GREGORIAN_STR);
+                break;
+            case CALTYPE_JULIAN:
+                strcpy(axis_units[k], CALTYPE_JULIAN_STR);
+                break;
+            case CALTYPE_ALLLEAP:
+                strcpy(axis_units[k], CALTYPE_ALLLEAP_STR);
+                break;
+            default:
+                sprintf(errmsg, "Unexpected calendar type of %d", calendar_type);
+                PyErr_SetString(PyExc_ValueError, errmsg);
+                Py_DECREF(axis_coords[k]);
+                while ( k > 0 ) {
+                    k--;
+                    Py_DECREF(axis_coords[k]);
+                }
+                Py_DECREF(badval_ndarray);
+                Py_DECREF(data_ndarray);
+                return NULL;
+            }
+            break;
+        case AXISTYPE_NORMAL:
+            /* axis normal to the results - no coordinates */
+            Py_INCREF(Py_None);
+            axis_coords[k] = Py_None;
+            axis_units[k][0] = '\0';
+            axis_names[k][0] = '\0';
+            break;
+        default:
+            sprintf(errmsg, "Unexpected axis type of %d", axis_types[k]);
+            PyErr_SetString(PyExc_RuntimeError, errmsg);
+            while ( k > 0 ) {
+                k--;
+                Py_DECREF(axis_coords[k]);
+            }
+            Py_DECREF(badval_ndarray);
+            Py_DECREF(data_ndarray);
+            return NULL;
+        }
+    }
+
+    /*
+     * Return a tuple (stealing references for PyObjects) of data_ndarray,
+     * axis_types, axis_names, axis_units, and axis_coords.
+     * Note: if MAX_FERRET_NDIM changes, this needs editing.
+     */
+    return Py_BuildValue("NN(iiiiii)(ssssss)(ssssss)(NNNNNN)", data_ndarray, badval_ndarray,
+              axis_types[0], axis_types[1], axis_types[2], axis_types[3], axis_types[4], axis_types[5],
+              axis_names[0], axis_names[1], axis_names[2], axis_names[3], axis_names[4], axis_names[5],
+              axis_units[0], axis_units[1], axis_units[2], axis_units[3], axis_units[4], axis_units[5],
+              axis_coords[0], axis_coords[1], axis_coords[2], axis_coords[3], axis_coords[4], axis_coords[5]);
+}
+
+
 static char pyferretStopDocstring[] =
     "Runs a series of Ferret commands to return Ferret to \n"
     "its default state, then shuts down and releases all \n"
@@ -1776,6 +2118,7 @@ static struct PyMethodDef pyferretMethods[] = {
     {"_start", (PyCFunction) pyferretStart, METH_VARARGS | METH_KEYWORDS, pyferretStartDocstring},
     {"_run", (PyCFunction) pyferretRunCommand, METH_VARARGS | METH_KEYWORDS, pyferretRunCommandDocstring},
     {"_get", (PyCFunction) pyferretGetData, METH_VARARGS | METH_KEYWORDS, pyferretGetDataDocstring},
+    {"_getstrdata", (PyCFunction) pyferretGetStrData, METH_VARARGS | METH_KEYWORDS, pyferretGetStrDataDocstring},
     {"_put", (PyCFunction) pyferretPutData, METH_VARARGS | METH_KEYWORDS, pyferretPutDataDocstring},
     {"_resize", (PyCFunction) pyferretResizeMemory, METH_VARARGS | METH_KEYWORDS, pyferretResizeMemoryDocstring},
     {"_stop", (PyCFunction) pyferretStop, METH_NOARGS, pyferretStopDocstring},
@@ -1846,9 +2189,20 @@ PyMODINIT_FUNC initlibpyferret(void)
     PyModule_AddIntConstant(mod, "TIMEARRAY_MINUTEINDEX", TIMEARRAY_MINUTEINDEX);
     PyModule_AddIntConstant(mod, "TIMEARRAY_SECONDINDEX", TIMEARRAY_SECONDINDEX);
 
+    /* Parameters for the calendar types */
+    PyModule_AddStringConstant(mod, "CALTYPE_360DAY", CALTYPE_360DAY_STR);
+    PyModule_AddStringConstant(mod, "CALTYPE_NOLEAP", CALTYPE_NOLEAP_STR);
+    PyModule_AddStringConstant(mod, "CALTYPE_GREGORIAN", CALTYPE_GREGORIAN_STR);
+    PyModule_AddStringConstant(mod, "CALTYPE_JULIAN", CALTYPE_JULIAN_STR);
+    PyModule_AddStringConstant(mod, "CALTYPE_ALLLEAP", CALTYPE_ALLLEAP_STR);
+    PyModule_AddStringConstant(mod, "CALTYPE_NONE", CALTYPE_NONE_STR);
+
     /* Parameter giving the maximum number of axis allowed in Ferret */
     PyModule_AddIntConstant(mod, "MAX_FERRET_NDIM", MAX_FERRET_NDIM);
 
+    /* Parameter giving the String used as the missing value for String arrays */
+    PyModule_AddStringConstant(mod, "STRING_MISSING_VALUE", STRING_MISSING_VALUE);
+
     /* Private parameter return value from libpyferret._run indicating the program should shut down */
     PyModule_AddIntConstant(mod, "_FERR_EXIT_PROGRAM", FERR_EXIT_PROGRAM);
 }
