@@ -48,6 +48,13 @@ typedef enum CALTYPE_ {
     CALTYPE_ALLLEAP = 60000,
 } CALTYPE;
 
+#define CALTYPE_360DAY_STR "CALTYPE_360DAY"
+#define CALTYPE_NOLEAP_STR "CALTYPE_NOLEAP"
+#define CALTYPE_GREGORIAN_STR "CALTYPE_GREGORIAN"
+#define CALTYPE_JULIAN_STR "CALTYPE_JULIAN"
+#define CALTYPE_ALLLEAP_STR "CALTYPE_ALLLEAP"
+#define CALTYPE_NONE_STR "CALTYPE_NONE"
+
 /* Prototypes for library C functions */
 void set_fer_memory(double *mem, size_t mem_size);
 void set_ppl_memory(float *mem, int mem_size);
@@ -79,6 +86,11 @@ void get_data_array_params_(char dataname[], int *lendataname, double *memory, i
                             int incr[MAX_FERRET_NDIM], char dataunit[], int *lendataunit,
                             AXISTYPE axtypes[MAX_FERRET_NDIM], double *badval, char errmsg[],
                             int *lenerrmsg, int maxlen_dataname, int maxlen_dataunit, int maxlen_errmsg);
+void get_str_data_array_params_(char dataname[], int *lendataname, double *memory, int *arraystart,
+                            int memlo[MAX_FERRET_NDIM], int memhi[MAX_FERRET_NDIM],
+                            int steplo[MAX_FERRET_NDIM], int stephi[MAX_FERRET_NDIM],
+                            int incr[MAX_FERRET_NDIM], AXISTYPE axtypes[MAX_FERRET_NDIM], char errmsg[],
+                            int *lenerrmsg, int maxlen_dataname, int maxlen_dataunit, int maxlen_errmsg);
 void get_data_array_coords_(double axiscoords[], char axisunit[], char axisname[],
                             int *axisnum, int *numcoords, char errmsg[], int *lenerrmsg,
                             int maxlen_axisunit, int maxlen_axisname, int maxlen_errmsg);
@@ -97,6 +109,9 @@ void proclaim_c_(int *ttoutLun, char *leader, int *quiet);
 void set_one_cmnd_mode_(int *one_cmnd_mode_int);
 void turnoff_verify_(int *status);
 
+/* Missing value for string arrays in Python - must be null-terminated string */
+#define STRING_MISSING_VALUE "\004\000"
+
 /* Functions for Python-backed external functions */
 
 #define INIT_METHOD_NAME "ferret_init"
